#!/usr/bin/env python
#
# Time Drive 0.2.0 - based on duplicity - Encrypted bandwidth efficient backup
# Requires Duplicity Version 0.6.04 released August 01, 2009.
#
# Copyright 2009 Rob Oakes	<LyX-Devel@oak-tree>

from PyQt4 import QtCore, QtGui

from ui_AdvancedFolderSettings import Ui_AdvancedFolderSettings
import utils
from backupsettings import backupfolder
from backupsettings import excludeitem
from backupsettings import globals

class AdvancedFolderSettings(QtGui.QDialog):
	"""Set advanced settings for archives included in the backup job description."""
	
	def __init__(self, parent):
		QtGui.QDialog.__init__(self, parent)
		self.ui = Ui_AdvancedFolderSettings()
		self.ui.setupUi(self)
		self.folder = None
		self.LoadSnapshots = self.parent().parent().LoadSnapshots
		self.ManageBackups = self.parent().parent().ManageBackups
		
		# Set Up the Snapshots List
		self.modelSnapshotsList = self.setupSnapshotsList()
		self.ui.treeSnapshotHistory.setModel(self.modelSnapshotsList)
		self.setupSnapshotsList_SetViewParameters()
		
		# Set Initial State of UI
		self.ui.lblTstResult.setVisible(False)
		
		# Events and Methods
		QtCore.QObject.connect(self.ui.radioGeneralSettings, QtCore.SIGNAL("released()"),
			self.ArchiveSettings_Update)
		QtCore.QObject.connect(self.ui.radioLocalBackup, QtCore.SIGNAL("released()"),
			self.ArchiveSettings_Update)
		QtCore.QObject.connect(self.ui.radioRemoteBackup, QtCore.SIGNAL("released()"),
			self.ArchiveSettings_Update)
		QtCore.QObject.connect(self.ui.btnExcludeAddPattern, QtCore.SIGNAL("clicked()"),
			self.btnExcludeAddPatern_Pressed)
		QtCore.QObject.connect(self.ui.btnExcludeAddFolder, QtCore.SIGNAL("clicked()"),
			self.btnExcludeAddFolder_Pressed)
		QtCore.QObject.connect(self.ui.btnExcludeAddFile, QtCore.SIGNAL("clicked()"),
			self.btnExcludeAddFile_Pressed)
		QtCore.QObject.connect(self.ui.btnExcludeRemove, QtCore.SIGNAL("clicked()"),
			self.btnExcludeRemove_Pressed)
		QtCore.QObject.connect(self.ui.buttonSelectLocalFolder, QtCore.SIGNAL("clicked()"),
			self.buttonSelectLocalFolder_Pressed)
		QtCore.QObject.connect(self.ui.buttonTestConnection, QtCore.SIGNAL("clicked()"),
			self.buttonTestConnection_Pressed)
		QtCore.QObject.connect(self.ui.btnOk, QtCore.SIGNAL("clicked()"),
			self.btnOk_Pressed)
		QtCore.QObject.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"),
			self.btnCancel_Pressed)
		QtCore.QObject.connect(self.ui.checkEncryptFiles, QtCore.SIGNAL("toggled(bool)"), 
			self.checkEncryptFiles_Toggled)
		QtCore.QObject.connect(self.ui.btnLoadHistory, QtCore.SIGNAL("clicked()"),
			self.LoadFolderSnapshots)
		QtCore.QObject.connect(self.ui.btnDeleteSnapshot, QtCore.SIGNAL("clicked()"),
			self.manageBackups_DeleteSet)
		
		QtCore.QObject.connect(self.ui.treeSnapshotHistory, 
			QtCore.SIGNAL("activated(QModelIndex)"), self.ManageSnapshots_Update)
		QtCore.QObject.connect(self.ui.treeSnapshotHistory, 
			QtCore.SIGNAL("clicked(QModelIndex)"), self.ManageSnapshots_Update)
		
		# Signals and Slots Related to Threads
		QtCore.QObject.connect(self.parent().TestConnectionUrl, 
			QtCore.SIGNAL("output_advanced(bool)"), self.ShowRemoteResults)
		
		QtCore.QObject.connect(self.LoadSnapshots,  
			QtCore.SIGNAL("ReturnSnapshotList_Folder(QStandardItemModel*)"), 
			self.LoadFolderSnapshots_SetModelList)
		QtCore.QObject.connect(self.LoadSnapshots,
			QtCore.SIGNAL("toggleControls(bool)"), self.LoadFolderSnapshots_ToggleControls)
		
		QtCore.QObject.connect(self.parent().parent().ui.windowRestore.LoadArchive,
			QtCore.SIGNAL("toggleControls(bool)"), self.LoadFolderSnapshots_ToggleControls)
			
		QtCore.QObject.connect(self.ManageBackups, QtCore.SIGNAL("toggleControls(bool)"),
			self.LoadFolderSnapshots_ToggleControls)
		QtCore.QObject.connect(self.ManageBackups, QtCore.SIGNAL("finished()"),
			self.manageBackups_ResetModel)
		
		# Run Setup Events
		self.ArchiveSettings_Update()
		self.ManageSnapshots_Update()
	
	
	def LoadPreferences(self, folder):
		"""Sets the dialog box control values from a valid folder data list."""
		self.ui.lblFolderName.setText("Advanced Settings - " + folder.FolderName)
		self.folder = folder
		
		if (folder.BackupType == folder.GENERALSETTINGS):
			self.ui.radioGeneralSettings.setChecked(True)
		elif folder.BackupType == folder.LOCAL:
			self.ui.radioLocalBackup.setChecked(True)
		elif folder.BackupType == folder.REMOTE:
			self.ui.radioRemoteBackup.setChecked(True)
		
		# Load Folder Exclude List
		excludeItemList = []
		trashIcon = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Remove-Snapshot.png"))
		self.ui.treeExclude.clear()
		for item in folder.ExcludeList:
			excludeItem = QtGui.QTreeWidgetItem([QtCore.QString(item.Item)])
			excludeItem.setIcon(0, trashIcon)
			excludeItemList.append(excludeItem)
		self.ui.treeExclude.insertTopLevelItems(0, excludeItemList)
		
		# Load Archive Settings
		
		self.ui.txtLocalFolder.setText(folder.LocalBackupDirectory)
		self.ui.comboRemoteProtocol.setCurrentIndex(folder.RemoteProtocolInt)
		self.ui.txtRemotePath.setText(folder.RemotePath)
		self.ui.txtUsername.setText(folder.Username)
		self.ui.txtPassword.setText(folder.Password)
		self.ui.checkEncryptFiles.setChecked(folder.EncryptionKeyEnabled)
		self.ui.txtEncryptionKey.setText(folder.EncryptionKey)
		self.ArchiveSettings_Update()
	
	
	def AddToExcludeList(self, excludePattern, excludeType):
		"""Adds the string, excludePattern, to the folder specific exclude list."""		

		if excludePattern == None:
			return

		e = excludeitem.ExcludeItem(excludePattern, excludeType)

		excludeIcon = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Remove-Snapshot.png"))
		excludeItem = QtGui.QTreeWidgetItem([QtCore.QString(excludePattern)])
		excludeItem.setIcon(0,excludeIcon)
		self.ui.treeExclude.addTopLevelItem(excludeItem)
		self.folder.ExcludeList.append(e)
	
	
	def btnExcludeAddPatern_Pressed(self):
		excludePattern = QtGui.QInputDialog.getText(self, "Exclude Pattern", "", QtGui.QLineEdit.Normal)
		if not excludePattern[1]:
			return
		
		try:
			pattern = utils.verify_ExcludePattern(excludePattern)
			if pattern == None:
				return

			if utils.isValidRegExp(pattern):
				self.AddToExcludeList(pattern, globals.REGEXP)
			else:
				QtGui.QMessageBox.warning(self, "Invalid", "Invalid regular expressions")

		except:
			QtGui.QMessageBox.warning(self, "Exception", "Error regular expressions")	
	
	
	def btnExcludeAddFolder_Pressed(self):
		excludePattern = QtGui.QFileDialog.getExistingDirectory(self, "", self.folder.FolderName)
		
		if excludePattern != "":
			self.AddToExcludeList(excludePattern, globals.FOLDER)
	
	
	def btnExcludeAddFile_Pressed(self):
		excludePatterns = QtGui.QFileDialog.getOpenFileNames(self, "", self.folder.FolderName)
		
		if excludePatterns != []:
			for pattern in excludePatterns:
				self.AddToExcludeList(pattern, globals.FILE)
	
	
	def btnExcludeRemove_Pressed(self):
		removePattern = self.ui.treeExclude.currentItem()
		curIndex = self.ui.treeExclude.indexOfTopLevelItem(removePattern)
	
		if removePattern!=None:
			for e in self.folder.ExcludeList:
				if removePattern.text(0) == e.Item:
					self.folder.ExcludeList.remove(e)
					continue
			
			toDelete = self.ui.treeExclude.takeTopLevelItem(curIndex)
			del curIndex, removePattern, toDelete
	
	
	def btnOk_Pressed(self):
		self.UpdateFolderSettings()
		self.close()
	
	
	def btnCancel_Pressed(self):
		self.close()
	
	
	def closeEvent(self, event):
		self.folder = None
		self.ui.lblTstResult.setVisible(False)
		self.ui.btnDeleteSnapshot.setEnabled(False)
		self.modelSnapshotsList.clear()
		self.setupSnapshotsList(self.modelSnapshotsList)
		self.setupSnapshotsList_SetViewParameters()
	
	
	def UpdateFolderSettings(self):
		"""Updates the folder specific advanced settings."""
		
		if self.ui.radioGeneralSettings.isChecked():
			self.folder.BackupType = self.folder.GENERALSETTINGS
		elif self.ui.radioLocalBackup.isChecked():
			self.folder.BackupType = self.folder.LOCAL
		elif self.ui.radioRemoteBackup.isChecked():
			self.folder.BackupType = self.folder.REMOTE
		
		self.folder.LocalBackupDirectory = self.ui.txtLocalFolder.text()
		self.folder.RemoteProtocolInt = self.ui.comboRemoteProtocol.currentIndex()
		self.folder.RemoteProtocol = self.ui.comboRemoteProtocol.currentText()
		self.folder.Username = self.ui.txtUsername.text()
		self.folder.Password = self.ui.txtPassword.text()
		self.folder.RemotePath = self.ui.txtRemotePath.text()
		self.folder.EncryptionKeyEnabled = self.ui.checkEncryptFiles.isChecked()
		self.folder.EncryptionKey = self.ui.txtEncryptionKey.text()	
	
	def buttonSelectLocalFolder_Pressed(self):
		folderName = QtGui.QFileDialog.getExistingDirectory(self)
		
		if folderName != "":
			self.ui.txtLocalFolder.setText(folderName)
	
	
	def buttonTestConnection_Pressed(self):
		archiveProtocol = self.ui.comboRemoteProtocol.currentText()
		archiveUsername = self.ui.txtUsername.text()
		archivePassword = self.ui.txtPassword.text()
		archiveFolder = self.ui.txtRemotePath.text()
		
		fullUrl = archiveProtocol + '://' + archiveUsername + ':' + archivePassword + '@' + archiveFolder
		self.ui.buttonTestConnection.setEnabled(False)
		self.parent().TestConnectionUrl.TestAdvancedUrl(fullUrl)
	
	
	def ShowRemoteResults(self, tstResult):
		self.ui.lblTstResult.setVisible(True)
		
		if tstResult == True:
			imageSuccess = QtGui.QPixmap(':/PNG/Resources/Dialog-Ok.png')
			self.ui.lblTstResult.setPixmap(imageSuccess)
		elif tstResult == False:
			imageFailure = QtGui.QPixmap(':/PNG/Resources/Dialog-Cancel.png')
			self.ui.lblTstResult.setPixmap(imageFailure)
		
		# Re-Enable buttonTestConnection
		self.ArchiveSettings_Update()
	
	
	def ArchiveSettings_Update(self):
		"""Method to enable/disable Archive Settings based on the current radio
		option selected.  When using global settings, local and remote should be
		disabled.  Likewise for when local or remote is enabled."""
		
		def LocalSettings(option):
			self.ui.txtLocalFolder.setEnabled(option)
			self.ui.buttonSelectLocalFolder.setEnabled(option)
		
		def RemoteSettings(option):
			self.ui.comboRemoteProtocol.setEnabled(option)
			self.ui.txtRemotePath.setEnabled(option)
			self.ui.buttonTestConnection.setEnabled(option)
			self.ui.lblUsername.setEnabled(option)
			self.ui.txtUsername.setEnabled(option)
			self.ui.lblPassword.setEnabled(option)
			self.ui.txtPassword.setEnabled(option)
		
		if self.ui.radioGeneralSettings.isChecked():
			LocalSettings(False)
			RemoteSettings(False)
		
		if self.ui.radioLocalBackup.isChecked():
			LocalSettings(True)
			RemoteSettings(False)
		
		if self.ui.radioRemoteBackup.isChecked():
			LocalSettings(False)
			RemoteSettings(True)
		
		self.checkEncryptFiles_Toggled()
	
	
	def checkEncryptFiles_Toggled(self):
		"""Enable/Disable the encryption options."""
		
		if self.ui.checkEncryptFiles.isChecked():
			self.ui.txtEncryptionKey.setEnabled(True)
		else:
			self.ui.txtEncryptionKey.setEnabled(False)
	
	
	def LoadFolderSnapshots(self):
		self.UpdateFolderSettings()
		backupSettings = self.parent().backupsettings
		self.LoadSnapshots.FolderList(backupSettings, self.folder)
		
		# Reset the Model and Ui Elements
		self.manageBackups_ResetModel()
		self.ui.btnDeleteSnapshot.setEnabled(False)
	
	
	def LoadFolderSnapshots_SetModelList(self, transferModel):
		"""Transfers the snapshots to the Snapshot model."""
		
		receiveModel = self.modelSnapshotsList
		iconFullBackup = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Full-Backup-Emblem.png"))
		currentFullBackup = receiveModel.invisibleRootItem()
		i = 0
		
		while transferModel.rowCount() > 0:
			backupType = transferModel.data(transferModel.index(0,3)).toString()
			moveRow = transferModel.takeRow(0)
			
			if backupType == "full":
				receiveModel.appendRow(moveRow)
				currentFullBackup = receiveModel.itemFromIndex(receiveModel.index(i,0))
				currentFullBackup.setIcon(iconFullBackup)
				i = i+1
				
			elif backupType == "inc":
				currentFullBackup.appendRow(moveRow)
		
		self.ui.treeSnapshotHistory.setColumnHidden(1, True)
		self.ui.treeSnapshotHistory.setColumnHidden(2, True)
	
	
	def LoadFolderSnapshots_ToggleControls(self, option):
		self.ui.btnLoadHistory.setEnabled(option)
		self.ui.btnRunAutoRemove.setEnabled(option)
	
	
	def setupSnapshotsList(self, currentModel = None):
		"""Sets the Horizontal Headers of the snapshotsModel, or creates
		a new one as necessary."""
		if currentModel == None:
			snapshotsList = QtGui.QStandardItemModel()
		else:
			snapshotsList = currentModel
		
		snapshotsList.setHorizontalHeaderItem(0, QtGui.QStandardItem("Date"))
		snapshotsList.setHorizontalHeaderItem(1, QtGui.QStandardItem("Timestamp"))
		snapshotsList.setHorizontalHeaderItem(2, QtGui.QStandardItem("Int"))
		snapshotsList.setHorizontalHeaderItem(3, QtGui.QStandardItem("Backup Type"))
		
		return snapshotsList
	
	
	def setupSnapshotsList_SetViewParameters(self):
		"""Hides the appropriate snapshotsModel Columns and sets the width
		of the others."""
		self.ui.treeSnapshotHistory.setColumnHidden(1, True)
		self.ui.treeSnapshotHistory.setColumnHidden(2, True)
		self.ui.treeSnapshotHistory.setColumnWidth(0, 350)
	
	
	def manageBackups_DeleteSet(self):
		"""Deletes the currently deleted snapshot."""
		curSelection = self.ui.treeSnapshotHistory.currentIndex()
		selectedBackup = self.modelSnapshotsList.data(curSelection.sibling(curSelection.row(),1)).toString()
		self.ManageBackups.DeleteBackupSet(selectedBackup, self.folder)
		self.ui.btnDeleteSnapshot.setEnabled(False)
	
	
	def manageBackups_ResetModel(self):
		self.modelSnapshotsList.clear()
		self.setupSnapshotsList(self.modelSnapshotsList)
		self.setupSnapshotsList_SetViewParameters()
	
	
	def ManageSnapshots_Update(self, currentIndex = None):
		"""Checks to see if a valid backup set is selected, if so, the 
		delete button is enabled.  If not, it is disabled."""
		
		try:
			currentItem = self.modelSnapshotsList.itemFromIndex(currentIndex)
			if currentItem != None:
				if (currentItem.index().isValid()) & (currentItem.parent() == None):
					self.ui.btnDeleteSnapshot.setEnabled(True)
				else:
					self.ui.btnDeleteSnapshot.setEnabled(False)
		except:
			self.ui.btnDeleteSnapshot.setEnabled(False)
