from PyQt4 import QtCore
import os
import timedrive.utils as utils
import backgroundtask
from timedrive.backupsettings import globals 
from timedrive.utils import duplicity_interface

class BackupJob(backgroundtask.BackgroundTask):
	"""Class that executes backup jobs and updates user on progress.  Designed to run
	as a baackground thread so that the main GUI instance remains responsive."""
	
	def __init__(self, parent = None):
		"""Initialization function for BackupJob"""
		super(BackupJob, self).__init__(parent)
		self.exiting = False
	
	def StartBackup(self, settings):
		"""Start a backup job, either from the GUI or from the command line."""
		self.settings = settings
		self.start()

	def isValidExclude(self, e):
		result  = False
		if e.Type == globals.FOLDER:
			if os.path.exists(e.Item):
				result = True
			else:
				print str(e.Item) + " could not be found, ignoring"

		elif e.Type == globals.FILE:
			if os.path.isfile(e.Item):
				result = True
			else:
				print str(e.Item) + " could not be found, ignoring"

		elif e.Type == globals.REGEXP:
			if utils.isValidRegExp(e.Item):
				result = True
			else:
				print str(e.Item) + " invalid REGEXP, ignoring"
		else:
			raise Exception("Invalid exclude type")
	
		return result
					
	def run(self):
		for folder in self.settings.FolderList:
			folderName = os.path.basename(str(folder.FolderName))
			archiveUrl, gnuPassphrase = utils.ArchiveUrl_FromSettings(folder, self.settings)
			combinedExcludeList = []
			
			if self.settings.Options_ExcludeHidden == True:
				combinedExcludeList.append(globals.EXCLUDEHIDDEN)
			
			if self.settings.Options_FullBackupFrequencyEnabled == True:
				FullBackupInterval = self.settings.Options_FullBackupFrequency
			else:
				FullBackupInterval = None
			
			for e in self.settings.ExcludeList:
				if self.isValidExclude(e):
					combinedExcludeList.append((e.Item, e.Type))

			for e in folder.ExcludeList:
				if self.isValidExclude(e):
					combinedExcludeList.append((e.Item, e.Type))
			
			try:
				# Start the Backup Job
				if self.settings.Options_EncryptFilesEnabled == False:
					gnuPassphrase = None
				
				if self.settings.AmazonS3_Location == "European":
					S3_BucketType = settings.AmazonS3_Location
				else:
					S3_BucketType = None
				
				if self.settings.AmazonS3Backup == True:
					result = duplicity_interface.start_backup(str(folder.FolderName), 
						archiveUrl, gnuPassphrase, combinedExcludeList, FullBackupInterval,
						self.settings.AmazonS3_AccessId, self.settings.AmazonS3_SecretKey,
						S3_BucketType)
				else:
					result = duplicity_interface.start_backup(str(folder.FolderName), 
						archiveUrl, gnuPassphrase, combinedExcludeList, FullBackupInterval)
	
				print 'Backup Successful: ' , folder.FolderName				
				print result
				
			except:
				message = QtCore.QString('Backup Failed: ' + folder.FolderName)
				print message
				self.taskError("Backup Job", message)
		
		self.taskComplete("Backup Job", "Backup Completed")