#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Time Drive 0.1.0 - based on duplicity - Encrypted bandwidth efficient backup
# Requires Duplicity Version 0.6.02 released July 08, 2009
#
# Copyright 2009 Rob Oakes	<LyX-Devel@oak-tree>

from PyQt4 import QtCore
from PyQt4 import QtGui
from advancedsettingsdialog import AdvancedFolderSettings
from background import testconnectionurl
from backupsettings import BackupSettings
from backupsettings import backupfolder
from backupsettings import excludeitem
from backupsettings import globals
import backupsettings
from ui_Settings import Ui_Settings
import utils

class SettingsDialog(QtGui.QDialog):
	"""Settings Dialog Class: Includes the methods for changing the global
	backup settings."""
	
	def __init__(self, parent, backupsettings):
		QtGui.QDialog.__init__(self, parent)
		self.ui = Ui_Settings()
		self.ui.setupUi(self)
		
		self.ui.lblTstResult.setVisible(False)
		
		self.RemoteResults = None
		self.LoadBackupSettings(backupsettings)
		
		self.TestConnectionUrl = testconnectionurl.TestConnectionUrl(self)
		self.ui.AdvancedFolderSettings = AdvancedFolderSettings(self)
		
		# Events and Methods
		QtCore.QObject.connect(self.ui.btnOk, QtCore.SIGNAL("clicked()"), 
			self.btnOk_Pressed)
		QtCore.QObject.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), 
			self.btnCancel_Pressed)
		QtCore.QObject.connect(self.ui.btnIncludeAdd, QtCore.SIGNAL("clicked()"), 
			self.btnIncludeAdd_Pressed)
		QtCore.QObject.connect(self.ui.btnIncludeRemove, QtCore.SIGNAL("clicked()"),
			self.btnIncludeRemove_Pressed)
		QtCore.QObject.connect(self.ui.buttonSelectLocalFolder, QtCore.SIGNAL("pressed()"),
			self.buttonSelectLocalFolder_Pressed)
		QtCore.QObject.connect(self.ui.buttonTestConnection, QtCore.SIGNAL("pressed()"),
			self.buttonTestConnection_Pressed)
		QtCore.QObject.connect(self.ui.btnExcludeAdd, QtCore.SIGNAL("clicked()"),
			self.btnExcludeAdd_Pressed)
		QtCore.QObject.connect(self.ui.btnExcludeRemove, QtCore.SIGNAL("clicked()"),
			self.btnExcludeRemove_Pressed)
		QtCore.QObject.connect(self.ui.buttonFolderOptions, QtCore.SIGNAL("clicked()"),
			self.buttonFolderOptions_Pressed)
		QtCore.QObject.connect(self.ui.buttonSelectDefaultRestoreLocation, QtCore.SIGNAL("pressed()"),
			self.buttonSelectDefaultRestoreLocation_Pressed)
		
		# Signals and Slots Related to Threads
		QtCore.QObject.connect(self.TestConnectionUrl, QtCore.SIGNAL("finished()"), self.ShowRemoteResults)
		QtCore.QObject.connect(self.TestConnectionUrl, QtCore.SIGNAL("terminated()"), self.ShowRemoteResults)
		QtCore.QObject.connect(self.TestConnectionUrl, QtCore.SIGNAL("output(bool)"), self.SetRemoteResults)
		
		# Events and Methods - General Tab
		QtCore.QObject.connect(self.ui.radioLocalBackup, QtCore.SIGNAL("released()"),
			self.GeneralSettings_Update)
		QtCore.QObject.connect(self.ui.radioRemoteBackup, QtCore.SIGNAL("released()"),
			self.GeneralSettings_Update)
		QtCore.QObject.connect(self.ui.radioAmazonS3Backup, QtCore.SIGNAL("released()"),
			self.GeneralSettings_Update)
		
		# Events and Methods - Auto Remove Tab
		QtCore.QObject.connect(self.ui.checkOlderThan, QtCore.SIGNAL("released()"), 
			self.AutoRemoveSettings_Update)
		QtCore.QObject.connect(self.ui.checkFreeSpace, QtCore.SIGNAL("released()"),
			self.AutoRemoveSettings_Update)
		
		# Events and Methods - Options Tab
		QtCore.QObject.connect(self.ui.checkEncryptFiles, QtCore.SIGNAL("released()"),
			self.Options_Update)
		QtCore.QObject.connect(self.ui.checkFullBackupFrequency, QtCore.SIGNAL("released()"),
			self.Options_Update)

		# Events and Methods - Advanced Tab
		QtCore.QObject.connect(self.ui.checkDefaultRestoreLocation, QtCore.SIGNAL("released()"),
			self.Advanced_Update)
	
	
	def LoadBackupSettings(self, backupsettings):
		self.backupsettings = backupsettings
		
		# Load General Options
		self.ui.radioLocalBackup.setChecked(backupsettings.LocalBackup)
		self.ui.txtLocalFolder.setText(backupsettings.LocalBackupDirectory)
		self.ui.radioRemoteBackup.setChecked(backupsettings.RemoteBackup)
		self.ui.comboRemoteProtocol.setCurrentIndex(backupsettings.RemoteProtocolInt)
		self.ui.txtRemotePath.setText(backupsettings.RemotePath)
		self.ui.txtUsername.setText(backupsettings.RemoteUsername)
		self.ui.txtPassword.setText(backupsettings.RemotePassword)
		self.ui.radioAmazonS3Backup.setChecked(backupsettings.AmazonS3Backup)
		self.ui.txtAmazonS3_Bucket.setText(backupsettings.AmazonS3_Bucket)
		self.ui.txtAmazonS3_AccessId.setText(backupsettings.AmazonS3_AccessId)
		self.ui.txtAmazonS3_SecretKey.setText(backupsettings.AmazonS3_SecretKey)
		self.ui.comboAmazonS3_BucketType.setCurrentIndex(backupsettings.AmazonS3_LocationInt)
		self.GeneralSettings_Update()
		
		# Load Auto-remove Options
		self.ui.checkOlderThan.setChecked(backupsettings.AutoRemove_OlderThan)
		self.ui.spinOlderThan.setValue(backupsettings.AutoRemove_OlderThan_Duration)
		self.ui.comboOlderThan.setCurrentIndex(backupsettings.AutoRemove_OlderThan_Units)
		self.ui.checkFreeSpace.setChecked(backupsettings.AutoRemove_FreeSpace)
		self.ui.spinFreeSpace.setValue(backupsettings.AutoRemove_FreeSpace_Size)
		self.ui.comboFreeSpace.setCurrentIndex(backupsettings.AutoRemove_FreeSpace_Units)
		self.ui.checkSmartRemove.setChecked(backupsettings.AutoRemove_SmartRemove)
		self.ui.checkPreserveNamedSnapshots.setChecked(backupsettings.AutoRemove_NamedSnapshots)
		self.AutoRemoveSettings_Update()
		
		# Load Backup Folder List (Include)
		includeItemList = []
		folderIcon = QtGui.QIcon(QtGui.QPixmap(":/SVG/Resources/Folder.svg"))
		self.ui.treeInclude.clear()
		for item in backupsettings.FolderList:
			"""for item in backupsettings.IncludeList.keys():"""
			includeItem = QtGui.QTreeWidgetItem([QtCore.QString(item.FolderName)])
			includeItem.setIcon(0,folderIcon)
			includeItemList.append(includeItem)
		self.ui.treeInclude.insertTopLevelItems(0, includeItemList)
		
		# Load Exclude List
		excludeItemList = []
		trashIcon = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Remove-Snapshot.png"))
		self.ui.treeExclude.clear()
		for item in backupsettings.ExcludeList:
			excludeItem = QtGui.QTreeWidgetItem([QtCore.QString(item.Item)])
			excludeItem.setIcon(0, trashIcon)
			excludeItemList.append(excludeItem)
		self.ui.treeExclude.insertTopLevelItems(0, excludeItemList)
		
		# Load "Options"
		self.ui.comboSchedule.setCurrentIndex(backupsettings.Schedule)
		self.ui.checkEncryptFiles.setChecked(backupsettings.Options_EncryptFilesEnabled)
		self.ui.txtEncryptionKey.setText(backupsettings.Options_EncryptionKey)
		self.ui.checkEnableNotifications.setChecked(backupsettings.Options_EnableNotifications)
		self.ui.checkExcludeHidden.setChecked(backupsettings.Options_ExcludeHidden)
		self.ui.checkFullBackupFrequency.setChecked(backupsettings.Options_FullBackupFrequencyEnabled)
    		self.ui.spinFullBackupFrequency.setValue(backupsettings.Options_FullBackupFrequency)
		self.Options_Update()

		# Load "Advanced"
		self.ui.checkDefaultRestoreLocation.setChecked(backupsettings.Advanced_DefaultRestoreEnabled)
		self.ui.txtDefaultRestoreLocation.setText(backupsettings.Advanced_DefaultRestoreLocation)
		self.Advanced_Update()
	
	def GeneralSettings_Update(self):
		"""Enable or disable the general settings controls based on the 
		options selected.  If Local Backup Directory is selected then remote 
		options should be disabled.  Likewise if the remote or options are selected."""
		
		def setLocalOptions(option):
			"""Enable/Disable for local backup"""
			self.ui.txtLocalFolder.setEnabled(option)
			self.ui.buttonSelectLocalFolder.setEnabled(option)
		
		def setRemoteOptions(option):
			"""Enable/Disable for remote backup"""
			self.ui.txtRemotePath.setEnabled(option)
			self.ui.lblUsername.setEnabled(option)
			self.ui.txtUsername.setEnabled(option)
			self.ui.lblPassword.setEnabled(option)
			self.ui.txtPassword.setEnabled(option)
			self.ui.comboRemoteProtocol.setEnabled(option)
			self.ui.buttonTestConnection.setEnabled(option)
		
		def setAmazonS3Options(option):
			"""Enable/Disable for Amazon S3"""
			self.ui.txtAmazonS3_Bucket.setEnabled(option)
			self.ui.lblAmazonS3_AccessId.setEnabled(option)
			self.ui.lblAmazonS3_BucketType.setEnabled(option)
			self.ui.txtAmazonS3_AccessId.setEnabled(option)
			self.ui.lblAmazonS3_SecretKey.setEnabled(option)
			self.ui.txtAmazonS3_SecretKey.setEnabled(option)
			self.ui.comboAmazonS3_BucketType.setEnabled(option)
			
		# Enable Local Backup
		if self.ui.radioLocalBackup.isChecked():
			setLocalOptions(True)
			setRemoteOptions(False)
			setAmazonS3Options(False)
		
		# Enable Remote Backup
		if self.ui.radioRemoteBackup.isChecked():
			setLocalOptions(False)
			setRemoteOptions(True)
			setAmazonS3Options(False)

		if self.ui.radioAmazonS3Backup.isChecked():
			setLocalOptions(False)
			setRemoteOptions(False)
			setAmazonS3Options(True)
	
	def AutoRemoveSettings_Update(self):
		"""Enable or disable the auto-remove settings based on the options selected."""
		
		def setOlderThan(option):
			"""Enable/Disable for older than options"""
			self.ui.spinOlderThan.setEnabled(option)
			self.ui.comboOlderThan.setEnabled(option)
		
		def setFreeSpace(option):
			"""Enable/Disable for free space options"""
			self.ui.spinFreeSpace.setEnabled(option)
			self.ui.comboFreeSpace.setEnabled(option)
		
		if self.ui.checkOlderThan.isChecked():
			setOlderThan(True)
		else:
			setOlderThan(False)
		
		if self.ui.checkFreeSpace.isChecked():
			setFreeSpace(True)
		else:
			setFreeSpace(False)
	
	
	def Options_Update(self):
		"""Enable or disable the options controls based on the values selected."""
		if self.ui.checkEncryptFiles.isChecked():
			self.ui.txtEncryptionKey.setEnabled(True)
		else:
			self.ui.txtEncryptionKey.setEnabled(False)
		
		if self.ui.checkFullBackupFrequency.isChecked():
			self.ui.lblFullBackupFrequency_Days.setEnabled(True)
			self.ui.spinFullBackupFrequency.setEnabled(True)
		else:
			self.ui.lblFullBackupFrequency_Days.setEnabled(False)
			self.ui.spinFullBackupFrequency.setEnabled(False)
	
	def Advanced_Update(self):
		"""Enable or disable the options controls based on the values selected."""
		self.ui.txtDefaultRestoreLocation.setEnabled(self.ui.checkDefaultRestoreLocation.isChecked())
		self.ui.buttonSelectDefaultRestoreLocation.setEnabled(self.ui.checkDefaultRestoreLocation.isChecked())
	
	def UpdateSettings(self):
		"""Reads settings and preferences from the settings dialog and writes them
		to platform appropriate storage.  (Registry on Windows, Xml Files on Mac OS X,
		and .ini files on Linux/Unix."""
		settings = BackupSettings()

		# Local Backup Settings
		settings.LocalBackup = self.ui.radioLocalBackup.isChecked()
		settings.LocalBackupDirectory = self.ui.txtLocalFolder.text()
		
		# Remote Backup Settings
		settings.RemoteBackup = self.ui.radioRemoteBackup.isChecked()
		settings.RemoteProtocolInt = self.ui.comboRemoteProtocol.currentIndex()
		settings.RemoteProtocol = self.ui.comboRemoteProtocol.currentText()
		settings.RemotePath = self.ui.txtRemotePath.text()
		settings.RemoteUsername = self.ui.txtUsername.text()
		settings.RemotePassword = self.ui.txtPassword.text()

		# Amazon S3
		settings.AmazonS3Backup = self.ui.radioAmazonS3Backup.isChecked()
		settings.AmazonS3_Bucket = self.ui.txtAmazonS3_Bucket.text()
		settings.AmazonS3_AccessId = self.ui.txtAmazonS3_AccessId.text()
		settings.AmazonS3_SecretKey = self.ui.txtAmazonS3_SecretKey.text()
		settings.AmazonS3_LocationInt = self.ui.comboAmazonS3_BucketType.currentIndex()
		settings.AmazonS3_Location = self.ui.comboAmazonS3_BucketType.currentText()
		
		# Autoremove Settings - Older Than
		settings.AutoRemove_OlderThan = self.ui.checkOlderThan.isChecked()
		settings.AutoRemove_OlderThan_Duration = self.ui.spinOlderThan.value()
		settings.AutoRemove_OlderThan_Units = self.ui.comboOlderThan.currentIndex()
		
		# Autoremove Settings - Free Space
		settings.AutoRemove_FreeSpace = self.ui.checkFreeSpace.isChecked()
		settings.AutoRemove_FreeSpace_Size = self.ui.spinFreeSpace.value()
		settings.AutoRemove_FreeSpace_Units = self.ui.comboFreeSpace.currentIndex()
		
		# Autoremove Settings - Smart Remove
		settings.AutoRemove_SmartRemove = self.ui.checkSmartRemove.isChecked()
		settings.AutoRemove_NamedSnapshots = self.ui.checkPreserveNamedSnapshots.isChecked()
		
		# Backup Schedule
		if settings.Schedule != self.ui.comboSchedule.currentIndex():
			utils.setup_cron( self.ui.comboSchedule.currentIndex())
		settings.Schedule = self.ui.comboSchedule.currentIndex()
		
		# Archive Options
		settings.Options_EncryptFilesEnabled = self.ui.checkEncryptFiles.isChecked()
		settings.Options_EncryptionKey = self.ui.txtEncryptionKey.text()
		settings.Options_EnableNotifications = self.ui.checkEnableNotifications.isChecked()
		settings.Options_ExcludeHidden = self.ui.checkExcludeHidden.isChecked()
		settings.Options_FullBackupFrequencyEnabled = self.ui.checkFullBackupFrequency.isChecked()
    		settings.Options_FullBackupFrequency = self.ui.spinFullBackupFrequency.value()

		# advanced
		settings.Advanced_DefaultRestoreEnabled = self.ui.checkDefaultRestoreLocation.isChecked()
		settings.Advanced_DefaultRestoreLocation = self.ui.txtDefaultRestoreLocation.text()

		backupsettings.writeSettings(settings)
	
	
	def btnOk_Pressed(self):
		self.UpdateSettings()			
		self.close()
	
	
	def btnCancel_Pressed(self):
		self.close()
		
	def closeEvent(self, event):
		self.ui.lblTstResult.setVisible(False)
	
	
	def btnIncludeAdd_Pressed(self):
		folderName = QtGui.QFileDialog.getExistingDirectory()
		
		if folderName != "":
			folderIcon = QtGui.QIcon(QtGui.QPixmap(":/SVG/Resources/Folder.svg"))
			includeItem = QtGui.QTreeWidgetItem([QtCore.QString(folderName)])
			includeItem.setIcon(0,folderIcon)
			self.ui.treeInclude.addTopLevelItem(includeItem)
			f = backupfolder.BackupFolder()
			f.FolderName = folderName
			f.BackupType = f.GENERALSETTINGS
			f.LocalBackupDirectory = ""
			f.RemoteProtocolInt = 0
			f.RemoteProtocol = ""
			f.RemotePath = ""
			f.Username = ""
			f.Password = ""
			f.RemotePath = ""
			f.EncryptionKeyEnabled = False
			f.EncryptionKey = ""
			self.backupsettings.FolderList.append(f)
	
	
	def btnIncludeRemove_Pressed(self):
		removeFolder = self.ui.treeInclude.currentItem()
		curIndex = self.ui.treeInclude.indexOfTopLevelItem(removeFolder)
		
		if removeFolder!=None:
			for f in self.backupsettings.FolderList:
				if removeFolder.text(0) == f.FolderName:
					self.backupsettings.FolderList.remove(f)
					continue

			toDelete = self.ui.treeInclude.takeTopLevelItem(curIndex)
			del curIndex, removeFolder, toDelete

	
	
	def buttonSelectLocalFolder_Pressed(self):
		folderName = QtGui.QFileDialog.getExistingDirectory()
			
		if folderName !="":
			self.ui.txtLocalFolder.setText(folderName)

	def buttonSelectDefaultRestoreLocation_Pressed(self):
		folderName = QtGui.QFileDialog.getExistingDirectory()
			
		if folderName !="":
			self.ui.txtDefaultRestoreLocation.setText(folderName)
			
	def buttonTestConnection_Pressed(self):
		archiveProtocol = self.ui.comboRemoteProtocol.currentText()
		archiveUsername = self.ui.txtUsername.text()
		archivePassword = self.ui.txtPassword.text()
		archiveFolder = self.ui.txtRemotePath.text()
		
		fullUrl = archiveProtocol + '://' + archiveUsername + ':' + archivePassword + '@' + archiveFolder
		self.ui.buttonTestConnection.setEnabled(False)
		self.TestConnectionUrl.TestUrl(fullUrl)
	
	
	def SetRemoteResults(self, tstResult):
		self.RemoteResults = tstResult
	
	
	def ShowRemoteResults(self):
		"""Updates the settings view based on the results of the connection test."""
		self.ui.lblTstResult.setVisible(True)
		
		if self.RemoteResults == True:
			imageSuccess = QtGui.QPixmap(':/PNG/Resources/Dialog-Ok.png')	
			self.ui.lblTstResult.setPixmap(imageSuccess)
		elif self.RemoteResults == False:
			imageFailure = QtGui.QPixmap(':/PNG/Resources/Dialog-Cancel.png')
			self.ui.lblTstResult.setPixmap(imageFailure)
		
		# Re-enable buttonTestConnection
		self.GeneralSettings_Update()
		self.RemoteResults = None
	
	
	def AddToExcludeList(self, excludePattern, excludeType):
		"""Adds the string, excludePattern, to the global exclude list."""
		excludeIcon = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Remove-Snapshot.png"))
		excludeItem = QtGui.QTreeWidgetItem([QtCore.QString(excludePattern)])
		excludeItem.setIcon(0,excludeIcon)
		self.ui.treeExclude.addTopLevelItem(excludeItem)
		e = excludeitem.ExcludeItem(excludePattern, excludeType)
		self.backupsettings.ExcludeList.append(e)
	
	
	def btnExcludeAdd_Pressed(self):
		excludePattern = QtGui.QInputDialog.getText(self, "Exclude Pattern", "", QtGui.QLineEdit.Normal)
		if not excludePattern[1]:
			return
		try:
			pattern = utils.verify_ExcludePattern(excludePattern)
			if pattern == None:
				return

			if utils.isValidRegExp(pattern):
				self.AddToExcludeList(pattern, globals.REGEXP)
			else:
				QtGui.QMessageBox.warning(self, "Invalid", "Invalid regular expressions")

		except:
			QtGui.QMessageBox.warning(self, "Exception", "Error regular expressions")		

			
	def btnExcludeRemove_Pressed(self):
		removePattern = self.ui.treeExclude.currentItem()
		curIndex = self.ui.treeExclude.indexOfTopLevelItem(removePattern)
		
		if removePattern!=None:
			for e in self.backupsettings.ExcludeList:
				if removePattern.text(0) == e.Item:
					self.backupsettings.ExcludeList.remove(e)
					continue

			toDelete = self.ui.treeExclude.takeTopLevelItem(curIndex)
			del curIndex, removePattern, toDelete
	
	
	def buttonFolderOptions_Pressed(self):
		if self.ui.treeInclude.currentItem() != None:
			folderName = self.ui.treeInclude.currentItem().text(0)
			if folderName != None:
				for f in self.backupsettings.FolderList:
					if folderName == f.FolderName:
						folder = f
						continue

				self.ui.AdvancedFolderSettings.LoadPreferences(folder)
				self.ui.AdvancedFolderSettings.show()
		else:
			QtGui.QMessageBox.warning(self, "Error", "No folder was selected!")
			
